%% This code runs the Monte Carlo analysis for Masten and Poirier (2019).
%
function BF_MC_main()
clear all;
clc;

%% 0. Set simulation grids
dgpGrid = [1];

nSimulations = 500;

sampleSizeGrid = [500 2000];

% NOTE: Must manually check that setOverallSimGrids.m also has the same
% grids set.

%% 1. Generate true and simulated datasets

% NOTE: This code uses the "makedist" function for defining the true dgp
% functions. versions of Matlab older then 2014a may not have this
% available.

for i=1:length(dgpGrid)

    % Compute and save true BFs for each dgp
    BF_illustration(i);
    
    % Generate all simulation datasets
    % This function sets the same seed each time it is called (and this
    % seed is a function of dgpChoice and sampleSize). Hence if we
    % want to increase nSimulations, the first part of the datasets will be
    % exactly the same and do not need to be computed again
    for j=1:length(sampleSizeGrid)
        sampleSize = sampleSizeGrid(j);
    
        generateMCdatasets(i,sampleSize,nSimulations);
    end

end

%% 2. Compute epsilon grid points

% For the next step, we need to know which epsilon's we're submitting. 
% To do this, we just scale up and down
%
% epsilonNaive = 1/sqrt(n);
%
% which gives us the naive bootstrap.
%
% We choose 
epsilonScaleGrid = [0.5 1 1.5 2 4 6 8 10];

epsilonGrid = nan(length(epsilonScaleGrid),length(sampleSizeGrid));

for i=1:length(sampleSizeGrid)
    epsilonNaive = ( 1/sqrt(sampleSizeGrid(i)) );
    
    for j=1:length(epsilonScaleGrid)
        epsilonGrid(j,i) = epsilonScaleGrid(j)*epsilonNaive;
    end
end

clear i j;

% These grid points are as follows:
%{
% For n=500,

epsilonNaive*0.5 = 0.0224 
epsilonNaive*1 = 0.0447
epsilonNaive*1.5 = 0.0671
epsilonNaive*2 = 0.0894
epsilonNaive*4 = 0.1789
epsilonNaive*6 = 0.2683
epsilonNaive*8 = 0.3578
epsilonNaive*10 = 0.4472

% For n=2000,

epsilonNaive*0.5 = 0.0112 
epsilonNaive*1 = 0.0224
epsilonNaive*1.5 = 0.0335
epsilonNaive*2 = 0.0447
epsilonNaive*4 = 0.0894
epsilonNaive*6 = 0.1342
epsilonNaive*8 = 0.1789
epsilonNaive*10 = 0.2236

%}

% For empirically calibrated dgp (for double bootstrap):

empiricalSampleSize = 448;
epsilonNaiveEmpirical = 1/sqrt(empiricalSampleSize);
epsilonGridEmpirical = nan(length(epsilonScaleGrid),1);
    
for j=1:length(epsilonScaleGrid)
    epsilonGridEmpirical(j) = epsilonScaleGrid(j)*epsilonNaiveEmpirical;
end

clear j empiricalSampleSize epsilonNaiveEmpirical;

%% 3. Compute estimates and confidence bands

% This step is done by submitting
%
% runMC_cluster(dgpChoice,sampleSize,nSimulations,epsilon)
%
% to the cluster in parallels.

%% 4. Compile Monte Carlo results

for k=1:length(dgpGrid)
    dgpChoice = dgpGrid(k);

    for i=1:length(sampleSizeGrid)
        sampleSize = sampleSizeGrid(i);

        for j=1:size(epsilonGrid,1)
            epsilon = epsilonGrid(j,i);
            
            fprintf('dgp %d n = %d eps = %2.4f\n',dgpChoice,sampleSize,epsilon);

            computeMCresults(dgpChoice,sampleSize,nSimulations,epsilon);

        end
    end
end
clear k i j;

%% 5. Compute empirical Monte carlo results

nSimuls = 500;

for j=1:size(epsilonGrid,1)
    epsilon = epsilonGridEmpirical(j);

    fprintf('Empirical simulations, eps = %2.4f\n',epsilon);

    computeEmpiricalMCresults(nSimuls,epsilon);

end

alphaGridEmpirical = [0.10 0.05];
plowerbarGridEmpirical = [0.1 0.25 0.5 0.75 0.9];
compileAllEmpiricalMCResults(alphaGridEmpirical,epsilonGridEmpirical,plowerbarGridEmpirical);

%% 5. Compile all MC results
% Here we just put the results together to make it easy for printing our
% tables

for k=1:length(dgpGrid)
    dgpChoice = dgpGrid(k);
    
    compileAllMCresults(dgpChoice);
end

%% 6. Print MC tables

printMCresults(1);
printMCresults(2);
printMCresults(3);
printMCresults(4);

end