%% This code performs the empirical analysis for Masten and Poirier (2019).
% It is executed by BF_empirical_RUN_EVERYTHING.m
function BF_empirical_main_v2(epsilon)

clc; 
display(epsilon);

rng(81349,'twister');
warning off optim:fminunc:SwitchingMethod;

generateMCcalibratedDatasets = 0;

baselineIsConditionalRankInvar = 1;

%% Set flags

Data.covariatesOn = 1;
params.covariatesOn = 1;
% There are two flags since not every function gets both params and Data
% passed to it all the time

if Data.covariatesOn ~= params.covariatesOn
    error('Covariate flags do not match.');
end

params.storeFbounds = 1; % Default is 1. This tells us to pre-compute
                         % Flower and Fupper on cgrid. Speeds up
                         % computations. But can't do this when computing
                         % ATE breakdown points, because that needs to work
                         % for arbitrary c values.

plotCDFsAndQuantiles = 0;
                         
%% Load data

run BF_empirical_loadData.m;
Data.Y = Y;
Data.W = [W1 W2];

% Relabel treatment, since P(Y1 >= Y0) = 0 under (tau,c) = (0,0).
% Thus 'treatment' here is NOT being abducted
Data.X = 1 - X;
clear Y X W1 W2;

Data.logWage = Data.Y;
Data.wage = exp(Data.Y);

%% Coarsen our original covariates
%
% sizeW1support = length(unique(Data.W(:,1))); % = 17
% sizeW2support = length(unique(Data.W(:,2))); % = 21

% numberOfCells = sizeW1support*sizeW2support; % = 357
%
% Since there are 357 cells, there isn't much data per covariate cell. 
%
% Moreover, we will take the covariate cells and the cross them with the two
% treatment cells. So if we coarsen each covariate into a binary variable,
% there are 4 covariate cells, giving us 8 cells total for our
% conditional cdfs.

Data.Woriginal = Data.W;

% Choose which coarsening to use
coarsenW = 6;

if coarsenW <= 3
    
    if coarsenW == 1
        % This is the baseline case

        quantile_val1 = 0.5;
        quantile_val2 = 0.5;

    elseif coarsenW == 2

        quantile_val1 = 0.75;
        quantile_val2 = 0.75;
        
    elseif coarsenW == 3

        quantile_val1 = 0.25;
        quantile_val2 = 0.25;
        
    end
    
    % New variable = 0 if below quantile_val1, = 1 if at or above that
    % quantile
    Data.W(:,1) = ( Data.W(:,1) >= quantile(Data.W(:,1),quantile_val1) );
    Data.W(:,2) = ( Data.W(:,2) >= quantile(Data.W(:,2),quantile_val2) );  
    
elseif coarsenW > 3
    
    if coarsenW == 4

        quantile_low_1 = 0.35;
        quantile_high_1 = 0.65;
        quantile_low_2 = 0.35;
        quantile_high_2 = 0.65;
        
    elseif coarsenW == 5

        quantile_low_1 = 0.25;
        quantile_high_1 = 0.75;
        quantile_low_2 = 0.25;
        quantile_high_2 = 0.75;
        
    elseif coarsenW == 6
 
        quantile_low_1 = 0.30;
        quantile_high_1 = 0.70;
        quantile_low_2 = 0.30;
        quantile_high_2 = 0.70;
        
    end
    
    cut_low_1 = quantile(Data.W(:,1),quantile_low_1);
    cut_high_1 = quantile(Data.W(:,1),quantile_high_1);

    cut_low_2 = quantile(Data.W(:,2),quantile_low_2);
    cut_high_2 = quantile(Data.W(:,2),quantile_high_2);
    
    A = 1.*(Data.W(:,1) <= cut_low_1) + ...
        2.*(Data.W(:,1) > cut_low_1).*(Data.W(:,1) <= cut_high_1) + ...
        3.*(Data.W(:,1) > cut_high_1);
    
    B = 1.*(Data.W(:,2) <= cut_low_2) + ...
        2.*(Data.W(:,2) > cut_low_2).*(Data.W(:,2) <= cut_high_2) + ...
        3.*(Data.W(:,2) > cut_high_2);
    
    Data.W(:,1) = A;
    Data.W(:,2) = B;
    
end
clear A B

% Display number of observations in each covariate cell
histcounts2(Data.W(:,1),Data.W(:,2))

%% Create a single covariate with 4 support points from the two binary ones

A = Data.W(:,1);
B = Data.W(:,2);

% WARNING: This only works if there are 9 or less groups for each variable
% Combine indices, so we get things like 11, 12, 21, 22
AB = arrayfun(@(x,y) str2double(strcat(num2str(x),num2str(y))),A,B);

% Create "index vector"
Data.Wsingle = grp2idx(AB);

Data.WsingleSuppSize = length(unique(Data.Wsingle));

for i=1:Data.WsingleSuppSize
    WsuppPoint = i;

    params.pW(i) = mean( (Data.Wsingle == WsuppPoint) );

end

clear A B AB WsuppPoint i;

%% Compute summary statistics
% This table presents summary stats for the original W, not the binary ones
% Likewise for the balance check table
printSummaryStats(Data);

%% Balance checks
% Here we simply recompute the summary statistics for Y,W1,W2, but now
% conditional on treatment

printNaiveComparisons(Data);

%% Estimate unconditional treatment probability

params.p1hat = mean(Data.X); % Probability of being treated
params.p0hat = 1 - params.p1hat;

params.sampleSize = length(Data.X);

%% Estimate conditional treatment probabilities, P(X=1 | W=w)
if params.covariatesOn == 1

    for i=1:Data.WsingleSuppSize
        WsuppPoint = i;

        params.p1hatCond(i) = mean(Data.X(Data.Wsingle == WsuppPoint));
        params.p0hatCond(i) = 1 - params.p1hatCond(i);

    end

    clear WsuppPoint i;

end

%% Estimate ATE

ATEconditional_logs = nan(Data.WsingleSuppSize,1);
ATEconditional_levels = nan(Data.WsingleSuppSize,1);

for k=1:Data.WsingleSuppSize
    WsuppPoint = k;
    
    treat = (Data.X == 1).*(Data.Wsingle == WsuppPoint);
    control = (Data.X == 0).*(Data.Wsingle == WsuppPoint);
    
    ATEconditional_logs(k) = mean(Data.logWage(treat==1)) - mean(Data.logWage(control==1));    
    ATEconditional_levels(k) = mean(Data.wage(treat==1)) - mean(Data.wage(control==1));
    
end

ATEhat_logs = params.pW*ATEconditional_logs;
ATEhat_levels = params.pW*ATEconditional_levels;

clear k treat control;

%% Compute standard errors for ATE under selection-on-observables
% E.g., see the first displayed equation at the top of page 322 of
% Hahn (1998) Econometrica

if params.covariatesOn == 1
    params.p1cond = params.p1hatCond;
    params.p0cond = params.p0hatCond;
end

% 1. Compute the conditional variances
sigma0sq_logs = nan(Data.WsingleSuppSize,1);
sigma1sq_logs = nan(Data.WsingleSuppSize,1);
sigma0sq_levels = nan(Data.WsingleSuppSize,1);
sigma1sq_levels = nan(Data.WsingleSuppSize,1);

for k=1:Data.WsingleSuppSize
    WsuppPoint = k;
    
    treat = (Data.X == 1).*(Data.Wsingle == WsuppPoint);
    control = (Data.X == 0).*(Data.Wsingle == WsuppPoint);
    
    sigma0sq_logs(k) = mean(Data.logWage(control==1).^2) - mean(Data.logWage(control==1))^2;
    sigma1sq_logs(k) = mean(Data.logWage(treat==1).^2) - mean(Data.logWage(treat==1))^2;
    
    sigma0sq_levels(k) = mean(Data.wage(control==1).^2) - mean(Data.wage(control==1))^2;
    sigma1sq_levels(k) = mean(Data.wage(treat==1).^2) - mean(Data.wage(treat==1))^2;

end

clear k treat control;

% 2. Compute the variance of the ATE estimator
ATEhat_var_logs = 0;
ATEhat_var_levels = 0;

% Here I just use the law of iterated expectations/total probability
for k=1:Data.WsingleSuppSize
    WsuppPoint = k;
    
    conditionalPiece_logs = ...
        ( (sigma1sq_logs(k)/params.p1cond(k)) + ...
          (sigma0sq_logs(k)/params.p0cond(k)) + ...
          (ATEconditional_logs(k) - ATEhat_logs)^2 )*params.pW(k);
      
    ATEhat_var_logs = ATEhat_var_logs + conditionalPiece_logs;

    conditionalPiece_levels = ...
        ( (sigma1sq_levels(k)/params.p1cond(k)) + ...
          (sigma0sq_levels(k)/params.p0cond(k)) + ...
          (ATEconditional_levels(k) - ATEhat_levels)^2 )*params.pW(k);
      
    ATEhat_var_levels = ATEhat_var_levels + conditionalPiece_levels;
end

clear k conditionalPiece_logs conditionalPiece_levels;

clear params.p1cond params.p0cond;

ATEhat_stdder_logs = sqrt(ATEhat_var_logs)/sqrt(params.sampleSize);
ATEhat_stdder_levels = sqrt(ATEhat_var_levels)/sqrt(params.sampleSize);

%% Compute bandwidths for unconditional (on W) cdfs

h1 = computeHansenBandwidth(Data.Y(Data.X == 1),4);
h0 = computeHansenBandwidth(Data.Y(Data.X == 0),4);

cdfBandwidth.x1 = h1/2;
% Here we manually undersmooth so as to better capture the local shape of
% the cdf, as can be seen visually

cdfBandwidth.x0 = h0;

clear h1 h0;

%% Compute bandwidths for conditional (on W) cdfs
if params.covariatesOn == 1
    
    for i=1:Data.WsingleSuppSize
        WsuppPoint = i;

        indices1 = ((Data.X == 1).*(Data.Wsingle == WsuppPoint) == 1);
        indices0 = ((Data.X == 0).*(Data.Wsingle == WsuppPoint) == 1);

        cdfBandwidth.x1cond(i) = ...
          computeHansenBandwidth(Data.Y(indices1),4);

        cdfBandwidth.x0cond(i) = ...
          computeHansenBandwidth(Data.Y(indices0),4);

    end

    % Again, we manually undersmooth to better capture the local shape, 
    % as can be seen visually
    cdfBandwidth.x1cond = cdfBandwidth.x1cond/2;
    cdfBandwidth.x0cond = cdfBandwidth.x0cond/2;

    clear indices1 indices0 WsuppPoint i;

end

% Store the positive bandwidth, because in some places we set this to zero
cdfBandwidthPositive = cdfBandwidth;

%% Set tuning parameters

%{

epsNaive = 1/sqrt(length(Data.Y));

0.5*epsNaive = 0.0236
1*epsNaive = 0.0472
1.5*epsNaive = 0.0709
2*epsNaive = 0.0945
4*epsNaive = 0.1890
6*epsNaive = 0.2835
8*epsNaive = 0.3780
10*epsNaive = 0.4725

%}

alphaGrid = [0.05];
alpha_ptIdentP1g0 = 0.05;
alpha_ATEbp = 0.05;

%% Set grid parameters

if params.covariatesOn == 1
    
    % Pointwise in w maximum
    cupperbarNatural = max([params.p1hatCond(:)' params.p0hatCond(:)']);
    
    % Here we manually set the upper bound, to speed up the bootstrap
    cupperbar = 0.25;
    
elseif params.covariatesOn == 0 
    
    cupperbar = max(params.p1hat,params.p0hat);
    
end

% Number of intervals used when finding roots in computing the integral for
% LBintegralPiece. This needs to be sufficiently large so that we pick up
% all true roots. Results can be very wrong if this is too small.
gridParams.Ngrid = 100;

% We are interested in P(Y1 - Y0 >= z) where z is:
z = 0;

% Number of grid points for computing the breakdown frontier
Ncgrid = 25;

cgrid = linspace(0,cupperbar,Ncgrid);

% These are used in UBintegralPiece.m for the bootstrap (used to do
% numerical integration)
gridParams.NgridIncrement = 40;
gridParams.NIntgrid = 500;

% Choose which claims about the parameter we are interested in: 
%
% P(Y1 >= Y0) >= plowerbar
%
plowerbarGrid = [0.1 0.25 0.5];

Nbootstrap = 1000;
Nbootstrap_ptIdentP1g0 = 1000;
Nbootstrap_ATEbp = 0;

fprintf(1,'Number of cgrid points = %d.\n',Ncgrid);
fprintf(1,'Number of BF bootstrap samples = %d.\n',Nbootstrap);

%% Generate datasets for bootstrap selection of epsilon
rng(230213,'twister');

if generateMCcalibratedDatasets == 1

DataOriginal = Data;
paramsOriginal = params;
clear Data params;

cdfBandwidth = cdfBandwidthPositive;
nSimulations = 500;

for i=1:nSimulations
    
    params.sampleSize = paramsOriginal.sampleSize;    
    U = rand(params.sampleSize,1);
    
    knot1 = paramsOriginal.pW(1);
    knot2 = sum(paramsOriginal.pW(1:2));
    knot3 = sum(paramsOriginal.pW(1:3));
    
    DataNew.Wsingle = 1*(U <= knot1) ...
            + 2*(knot1 < U).*(U <= knot2) ...
            + 3*(knot2 < U).*(U <= knot3) ...
            + 4*(U > knot3);
        
    V = rand(params.sampleSize,1);
    
    DataNew.X = (V <= paramsOriginal.p1hatCond(DataNew.Wsingle)');
    % We do NOT have to re-define treatment as 1-X in these generated
    % datasets, since we are drawing from P(X|W) where X is the correct
    % treatment variable of interest
        
    Z = rand(params.sampleSize,1);
    
    DataNew.Y = nan(params.sampleSize,1);
    
    for k=1:params.sampleSize
    
    % Probability integral transform
    DataNew.Y(k) = ...
        smoothedQuantile(Z(k),DataNew.X(k),DataOriginal,cdfBandwidth,DataNew.Wsingle(k));
    
    end
    
    Data = DataNew;
    clear U V Z DataNew;

    saveFileName = sprintf('../datasets/MCdatasetEmpirical_sim%d.mat',i);
    save(saveFileName,'Data','params');

end

Data = DataOriginal;
params = paramsOriginal;
clear DataOriginal paramsOriginal;

end

%% Estimate breakdown frontier, relaxing unconditional rank invariance
    
params.p1 = params.p1hat;
params.p0 = params.p0hat;

if params.covariatesOn == 1
    params.p1cond = params.p1hatCond;
    params.p0cond = params.p0hatCond;
end

if baselineIsConditionalRankInvar == 0

[BFvec,intPieceVec,minPieceVec] = ...
    computeBFvec(cgrid,plowerbarGrid,params,gridParams,z,Data,cdfBandwidth);

clear params.p1 params.p0 params.p1cond params.p0cond;

trunc.top = 0;
trunc.bottom = 0;
BCvec = computeBCvec(intPieceVec,minPieceVec,plowerbarGrid,trunc);

end

%% Estimate breakdown frontier, relaxing conditional rank invariance
if baselineIsConditionalRankInvar == 1
    
% Define vecs to hold the intPiece and minPiece over the covars
intPieceVecWcell = nan(length(cgrid),4);
minPieceVecWcell = nan(length(cgrid),4);

% Loop over the values of the covariates
for i=1:Data.WsingleSuppSize
    w = i;

    % Define new "Data" variable for a single covariate cell
    DataWcell.covariatesOn = 0;
    DataWcell.Y = Data.Y(Data.Wsingle==w);
    DataWcell.X = Data.X(Data.Wsingle==w);

    % Define new cdfBandwidth
    cdfBandwidthWcell.x1 = cdfBandwidth.x1cond(i);
    cdfBandwidthWcell.x0 = cdfBandwidth.x0cond(i);
    
    % Define new params
    paramsWcell.covariatesOn = 0;
    paramsWcell.storeFbounds = 1;
    paramsWcell.sampleSize = length(DataWcell.Y);
    paramsWcell.p1hat = params.p1hatCond(i);
    paramsWcell.p0hat = params.p0hatCond(i);
    paramsWcell.p1 = paramsWcell.p1hat;
    paramsWcell.p0 = paramsWcell.p0hat;

    % Get the intPiece and minPiece for that covariate cell
    [~,intPieceVecWcell(:,i),minPieceVecWcell(:,i)] = ...
        computeBFvec(cgrid,plowerbarGrid,paramsWcell,gridParams,z,DataWcell,cdfBandwidthWcell);

end
clear i DataWcell cdfBandwidthWcell paramsWcell;

% Average over the intPiece and minPiece to get the unconditional DTE
% bounds (i.e., law of total probability)
intPieceVec = intPieceVecWcell * params.pW';
minPieceVec = minPieceVecWcell * params.pW';

clear intPieceVecWcell minPieceVecWcell;

% Compute the BF based on the averaged intPiece and minPiece
trunc.top = 0;
trunc.bottom = 0;
BCvec = computeBCvec(intPieceVec,minPieceVec,plowerbarGrid,trunc);

trunc.top = 1;
trunc.bottom = 1;
BFvec = computeBCvec(intPieceVec,minPieceVec,plowerbarGrid,trunc);

end

%% Bootstrap
rng(130913,'twister');

if Nbootstrap > 0
        
    % Use empirical cdf for bootstrap draws
    % These must be set to zero even if we have covariates in the analysis
    cdfBandwidth.x1 = 0;
    cdfBandwidth.x0 = 0;
    
    if params.covariatesOn == 1
        for i=1:Data.WsingleSuppSize
            cdfBandwidth.x1cond(i) = 0;
            cdfBandwidth.x0cond(i) = 0;
        end
    end

    [BFvecBootCentered, BFvecBoot, BCvecBoot, BCvecBootCentered, ...
      intPieceVecBoot, minPieceVecBoot, minPieceVecBootNotMono] ...
        = computeBootstrapVecs(params,gridParams,z,cdfBandwidth,cgrid,...
                  plowerbarGrid,Data,BFvec,BCvec,Nbootstrap,epsilon,trunc,baselineIsConditionalRankInvar);
              
    params.useBC = 1; % Decide how to compute CIs: based on the BC or BF
    
    % The difference here is whether we use the BC or BF as our baseline
    if params.useBC == 1
        vecToPass = BCvec;
        validCenteredVec = BCvecBootCentered;
    elseif params.useBC == 0
        vecToPass = BFvec;
        validCenteredVec = BFvecBootCentered;
    end
    
    for i=1:length(alphaGrid)
        alpha = alphaGrid(i);

        CI(i) = computeBootstrapCI(params,cgrid,plowerbarGrid,...
                              vecToPass,validCenteredVec,Nbootstrap,alpha);
    end

elseif Nbootstrap == 0
    
    for i=1:length(alphaGrid)
        
        CI(i).BFvec_CIuniformUnequal_onesided_lb = ...
            zeros(length(cgrid),length(plowerbarGrid));
        
    end
    
end

%% Estimate parameter under point identification, (tau,c) = (0,0)
rng(018321,'twister');

% Note: Here we use empirical cdfs, just like in the BF bootstrap above,
% because it is much faster than smoothed cdfs.

params.p1 = params.p1hat;
params.p0 = params.p0hat;

if params.covariatesOn == 1
    params.p1cond = params.p1hatCond;
    params.p0cond = params.p0hatCond;
end

if baselineIsConditionalRankInvar == 0

ptIdentP1g0 = ...
    1-UBintegralPiece(params,gridParams,0,0,Data,cdfBandwidth);

elseif baselineIsConditionalRankInvar == 1
    
    ptIdentP1g0_Wcell = nan(1,4);

    % Loop over the values of the covariates
    for i=1:Data.WsingleSuppSize
        w = i;

        % Define new "Data" variable for a single covariate cell
        DataWcell.covariatesOn = 0;
        DataWcell.Y = Data.Y(Data.Wsingle==w);
        DataWcell.X = Data.X(Data.Wsingle==w);

        % Define new cdfBandwidth
        cdfBandwidthWcell.x1 = cdfBandwidth.x1cond(i);
        cdfBandwidthWcell.x0 = cdfBandwidth.x0cond(i);

        % Define new params
        paramsWcell.covariatesOn = 0;
        paramsWcell.storeFbounds = 1;
        paramsWcell.sampleSize = length(DataWcell.Y);
        paramsWcell.p1hat = params.p1hatCond(i);
        paramsWcell.p0hat = params.p0hatCond(i);
        paramsWcell.p1 = paramsWcell.p1hat;
        paramsWcell.p0 = paramsWcell.p0hat;
        
        ptIdentP1g0_Wcell(i) = 1-UBintegralPiece(paramsWcell,gridParams,0,0,DataWcell,cdfBandwidthWcell);

    end
    clear i DataWcell cdfBandwidthWcell paramsWcell;
    
    ptIdentP1g0 = ptIdentP1g0_Wcell * params.pW'; % = 0.6839

end

cdfBandwidth.x1 = 0;
cdfBandwidth.x0 = 0;

if params.covariatesOn == 1
    for i=1:Data.WsingleSuppSize
        cdfBandwidth.x1cond(i) = 0;
        cdfBandwidth.x0cond(i) = 0;
    end
end

if Nbootstrap_ptIdentP1g0 > 0

    % Standard nonparametric bootstrap
    % (But implemented as our numerical delta method with eps=1/sqrt(n)
    % only because that code is optimized for speed)
CI_ptIdentP1g0 = computeBootstrapCI_ptIdentP1g0(params,...
    gridParams,0,0,Data,cdfBandwidth,Nbootstrap_ptIdentP1g0,...
    alpha_ptIdentP1g0,Data.WsingleSuppSize,baselineIsConditionalRankInvar);

end    

clear params.p1 params.p0;
    
%% Estimate ATE breakdown point
rng(1239810,'twister');

% Note: The bootstrap here is slow because we can't pre-store
% Flower and Fupper because it has to work for many values of c (that's the
% variable we're searching over). So it's about 3--4 minutes per bootstrap
% draw.

params.p1 = params.p1hat;
params.p0 = params.p0hat;

if params.covariatesOn == 1
    params.p1cond = params.p1hatCond;
    params.p0cond = params.p0hatCond;
end

mugrid = [0];

ATEbreakdownPoints = computeATEbp(mugrid,params,cgrid,Data,cdfBandwidth);

if Nbootstrap_ATEbp > 0

% Use empirical cdf for bootstrap draws
cdfBandwidth.x1 = 0;
cdfBandwidth.x0 = 0;

if params.covariatesOn == 1
    for i=1:Data.WsingleSuppSize
        cdfBandwidth.x1cond(i) = 0;
        cdfBandwidth.x0cond(i) = 0;
    end
end

params.storeFbounds = 0;

epsilon_ATEbp = (1/sqrt(length(Data.Y))); % For ATE breakdown point the
                                          % naive bootstrap actually is
                                          % valid.

[CI_ATEbp,ATEbpBootCentered,ATEbpBoot] = computeBootstrapCI_ATEbp(mugrid,params,cgrid,Data,...
    ATEbreakdownPoints,cdfBandwidth,Nbootstrap_ATEbp,...
    epsilon_ATEbp,alpha_ATEbp);

CI_ATEbp_trunc = nan(length(mugrid),1);

% Truncate CI if necessary
for i=1:length(mugrid)
    if CI_ATEbp(i) < 0
        CI_ATEbp_trunc(i) = 0;
    elseif CI_ATEbp(i) >= 0
        CI_ATEbp_trunc(i) = CI_ATEbp(i);
    end
end

clear params.p1 params.p0;

end

%% Save output
save(sprintf('../output/BF_empirical_Nint%2.2d_Nc%2.2d_B%2.2d_covars%g_coarsen%g_eps%2.4f.mat',...
    gridParams.Ngrid,Ncgrid,Nbootstrap,params.covariatesOn,coarsenW,epsilon));

%% Plot BFs
extendPlots = 1;

for j=1:length(plowerbarGrid)
    plowerbar = plowerbarGrid(j);
    
    if extendPlots == 0
        BF = BFvec(:,j);
        BF_l = CI.BFvec_CIuniformUnequal_onesided_lb(:,j);
        cgridForPlots = cgrid;
        
    elseif extendPlots == 1
        
        BF = [BFvec(:,j)' 0];
        BF_l = [CI.BFvec_CIuniformUnequal_onesided_lb(:,j)' 0];
        cgridForPlots = [cgrid cupperbarNatural];
        
    end

        h=figure('visible','off');
        plot(cgridForPlots,BF,'LineWidth',2,'Color','black');
        hold on;
        plot(cgridForPlots,BF_l,'--','LineWidth',2,'Color','black');
        set(gca,'fontsize',15);
        title(strcat('Breakdown frontier, $\underline{p}$ = ',...
            sprintf('%2.2g',plowerbar)),'Interpreter','latex');
        xlabel('Relaxation of full independence');
        ylabel('Relaxation of rank invariance');
        box off;
        axis([0,cupperbarNatural,0,1]);
        print(h,'-deps',...
            sprintf('../images/BF_empirical_plowerbar%2.2g_Nint%2.2d_Nc%2.2d_B%2.2d_covars%g_coarsen%g_eps%2.4f.eps',...
                plowerbar,gridParams.Ngrid,Ncgrid,Nbootstrap,params.covariatesOn,coarsenW,epsilon));

        hold off;

end

end