*******************************************************************************
* Consequences.DataToMatlab.do
*******************************
*
* AUTHOR: Margaux Luflade
* DATE CREATED: 9.17.2016
* DATE LAST MODIFIED: 1.25.2018 (by Matt)
*
* This file:
* 1. Reads the SWAY.Replication.June2013 data file
* 2. Define variables needed in the analysis
* 3. Export the data to MATLAB. Specifically, it exports 3 main matrices:
*
* 	i. TREATMENT: abduction indicator; militia indicator (see columns in table 2
*	p.887) + top 4 variables in table 1 p.884 (abduction length; abduction age; 
*	index of violence experienced; index of violence perpetrated)
*
*	ii. OUTCOMES: bottom 10 table 1 p.884, in same order + log of monthly wage 
*	last
*
* 	iii. PRE-TREATMENT COVARIATES: see table 2 p.887, in same order

********************************************************************************
 
clear
set mem 40m
set matsize 400
set more off
cap log close

// INSTRUCTIONS: Modify this cd to be
// cd "YOUR DIRECTORY/data/SWAY.Replication.June2013/"
cd "/Users/matt/Desktop/BFemppirical/data/SWAY.Replication.June2013/"


//cd "YOUR DIRECTORY\MATLAB\SWAY\data\" = Destination of .csv files
global matlab "../data"

* 1) DATA PREPARATION
** This is copied from Blattman and Annan's replication file Consequences.Replication.June2013.do

// SORT DATASETS //

use Data/par_grp, clear
sort camp parish
save Data/par_grp, replace

use Data/abd_risk, clear
sort par_grp age
save Data/abd_risk, replace

use Data/abn_risk_total, clear
sort par_grp
save Data/abn_risk_total, replace

use Data/SWAY_I_June2013.dta, clear

// Create global macros
global depvar1 "literate emp_mo prof distress75 hostile fight"
global depvar2 "educ lwage_mo distress support"
global AC "A14-A29 C_ach-C_paj" 
*global ACG "A14-A29 C_ach-C_paj G1_18-G1_22 G2_14 G2_22 G3_14-G3_18 G4_14-G4_22 G5_14-G5_22 G6_14-G6_22 G7_14-G7_22 G8_14-G8_22" 
global ACG "A14-A29 C_ach-C_paj G1_14_17 G1_18_21 G2_14_17 G2_18_21 G3_14_17 G4_14_17 G4_18_21 G5_14_17 G5_18_21 G6_14_17 G6_18_21 G7_14_17 G7_18_21 G8_14_17 G8_18_21" 
global ctrls "fthr_ed0 fthr_ed4 fthr_ed7 mthr_ed0 mthr_ed4 mthr_ed7 no_fthr96 no_mthr96 orphan96 hh_fthr_frm hh_size96 hh_size96_2 hh_size96_3 hh_size96_4 hh_land hh_land_2 hh_land_3 hh_land_4 hh_cattle hh_cattle_2 hh_cattle_3 hh_cattle_4 hh_stock hh_stock_2 hh_stock_3 hh_stock_4 hh_plow"
global ctrl_hh "age hh_fthr_frm hh_size96 hh_land landrich hh_cattle hh_stock hh_plow"
global ctrl_i "fthr_ed0 fthr_ed mthr_ed0 mthr_ed no_fthr96 no_mthr96 orphan96"

// Create new variables
gen abd2w=cond(abd==1 & abd_lgth>.5,1,0)
gen landrich=cond(hh_land>=50 & hh_land!=.,1,0)
gen distress75=cond(distress>5,1,0) if found==1
gen distress90=cond(distress>=7,1,0) if found==1
gen quarrel=viii13/3
gen steal=viii18/3
gen curse=viii25/3
gen threaten=viii30/3
gen hostile2=cond(steal>0 | threaten>0 | quarrel>0,1,0) if found==1
gen mphone=cond(ii18k>0 & ii18k!=.,1,0)
gen radio=cond(ii18e>0 & ii18e!=.,1,0)
gen bike=cond(ii18c>0 & ii18c!=.,1,0)
gen lwage_mo = ln(wage_mo)
gen town_out_dist = cond(town==1 & out_district==1,1,0) // MGX_9/17: out_dist do not exist >> change into out_district

foreach var of varlist age hh_size96 hh_land landrich hh_cattle hh_stock hh_plow {
   gen `var'_2 = `var'^2
   gen `var'_3 = `var'^3
   gen `var'_4 = `var'^4
}

   //replace abd_l=abd_l/12 // MGX_9/17 this gives abduction length in years while in tables want it in months

// Create selection weights for use with abducted youth only

drop pscore*
logit abd $AC $ctrls [pw=w_abs2]
	predict pscore if found==1
gen z=w_abs2/pscore if abd==1 & found==1
egen sumz=sum(z) if found==1
   egen sumi=sum(i_weight)
gen w_abd=(z*sumi/sumz) if found==1
drop z sumz sumi

save Data/tempdata/consequences_temp.dta, replace

********************************************************************************

* 2) EXPORT TO MATLAB

use "Data/tempdata/consequences_temp.dta", clear

codebook found // no missing 
preserve
	keep id found 

	export delimited found ///
		using ${matlab}SWAY_found.csv, novarnames nolabel replace
restore


** TREATMENTS
br id abd army abd_l abd_age viol_experienced viol_perp
codebook abd // no missing 
codebook army // 475 missing
codebook abd_l // 197 missing .
codebook abd_age // 488 missing .
codebook viol_experienced // 478 missing .
codebook viol_perp // 478 missing .

preserve
	keep id abd army abd_l abd_age viol_experienced viol_perp 
	mvencode abd_l army abd_age viol_experienced viol_perp, mv(.=-1) 

	export delimited abd army abd_l abd_age viol_experienced viol_perp ///
		using ${matlab}SWAY_treatment.csv, novarnames nolabel replace
restore

** OUTCOMES
br id educ literate emp_mo prof wage_mo distress distress75 support ///
	hostile fight lwage_mo
codebook educ // 346 missing . 
codebook literate // 346 missing .
codebook emp_mo // no missing .
codebook prof // 475 missing .
codebook wage_mo // 712 missing .
codebook distress // no missing . 
codebook distress75 // 475 missing .
codebook support // no missing .
codebook hostile // 475 missing .
codebook fight // 475 missing .
codebook lwage_mo // 768 missing

preserve
	keep id educ literate emp_mo prof wage_mo distress distress75 support ///
		hostile fight lwage_mo
	mvencode educ literate emp_mo prof wage_mo distress distress75 support ///
		hostile fight lwage_mo, mv(.=-1) 

	export delimited educ literate emp_mo prof wage_mo distress distress75 ///
		support hostile fight lwage_mo ///
		using ${matlab}SWAY_outcomes.csv, novarnames nolabel replace
restore

** COVARIATES
br id age hh_fthr_frm hh_size96 hh_land landrich hh_cattle ///
		hh_stock hh_plow fthr_ed0 fthr_ed mthr_ed0 mthr_ed no_fthr96 no_mthr96 ///
		orphan96
codebook age // no missing . 
codebook hh_fthr_frm // no missing .
codebook hh_size96 // no missing .
codebook hh_land // no missing .
codebook landrich // no missing .
codebook hh_cattle // no missing .
codebook hh_stock // no missing . 
codebook hh_plow // no missing .
codebook fthr_ed0 // 475 missing .
codebook fthr_ed // 475 missing .
codebook mthr_ed0 // 475 missing .
codebook mthr_ed // 475 missing .
codebook no_fthr96 // 475 missing .
codebook no_mthr96 // 475 missing .
codebook orphan96 // 475 missing .
		
preserve
	keep id age hh_fthr_frm hh_size96 hh_land landrich hh_cattle ///
		hh_stock hh_plow fthr_ed0 fthr_ed mthr_ed0 mthr_ed no_fthr96 no_mthr96 ///
		orphan96
	mvencode fthr_ed mthr_ed0 mthr_ed no_fthr96 no_mthr96 orphan96, mv(.=-1)

	export delimited age hh_fthr_frm hh_size96 hh_land landrich hh_cattle ///
		hh_stock hh_plow fthr_ed0 fthr_ed mthr_ed0 mthr_ed no_fthr96 no_mthr96 ///
		orphan96 ///
		using ${matlab}SWAY_covariates.csv, novarnames nolabel replace
restore
