% Here we compute int_0^1 indicator[f(u) <= 0] du.
%
% We use two ways of computing it:
%
% 1. Numerical integration of a smoothed indicator.
%
% 2. Root finding approach.
%
% Approach 2 works by computing all roots of f and using this to partition 
% the domain appropriately into the regions where f is positive and the
% ones where it is negative. This approach is very fast, but it is not 
% robust if f(u) is very wiggly. In contrast, approach 1 is slower, but is
% robust when f(u) is wiggly.
%
% So we will switch between methods 1 and 2. We use method 2 for computing
% the true and estimated curves. We use method 1 for bootstrap curve
% estimates (because the bootstrap can lead to wiggles).
function value = UBintegralPiece(params,gridParams,z,c,...
                                    Data,h,DataBootstrap,epsilon,forceNumericalInt)
                                                     
    if nargin == 4
        fun = @(u) (Qlower(params,u,1,c) - Qupper(params,u,0,c) - z);
        
        useNumericalIntegration = 0;
        
    elseif nargin == 6
        fun = ...
           @(u)(Qlower(params,u,1,c,Data,h)-Qupper(params,u,0,c,Data,h)-z);
       
        useNumericalIntegration = 0;
        
    elseif nargin == 8
        fun = ...
           @(u)(Qlower(params,u,1,c,Data,h,DataBootstrap,epsilon)...
                -Qupper(params,u,0,c,Data,h,DataBootstrap,epsilon)-z);
            
        useNumericalIntegration = 1;
        
    elseif nargin == 9
        % There is only one case where nargin=9: When nargin=6 originally, 
        % but then because of flat spots in fun near a root, 
        % UBintegralPiece calls itself recursively and switches to the
        % numerical integration method, which is much more robust.
        fun = ...
           @(u)(Qlower(params,u,1,c,Data,h)-Qupper(params,u,0,c,Data,h)-z);
       
        useNumericalIntegration = 1;
        
    end
    
    if useNumericalIntegration == 1
        
        NIntgrid = gridParams.NIntgrid;
        Ugrid = linspace(0,1,NIntgrid);

        useSmoothIndicator = 1;
        smoothingBandwidth = 0.001;

        integrand = nan(length(Ugrid),1);
        for i=1:length(Ugrid)

            if useSmoothIndicator == 0     

                integrand(i) = (fun(Ugrid(i)) <= 0);

            elseif useSmoothIndicator == 1

                insidePiece = fun(Ugrid(i));

                integrand(i) = logitKernel(-insidePiece,smoothingBandwidth);

            end

        end

        value = mean(integrand);
        
    elseif useNumericalIntegration == 0
    
    startingPoints = linspace(0,1,gridParams.Ngrid);
    
    roots = [];    
    for i = 1:(gridParams.Ngrid-1)
        try
            % Here we look for a root only within the interval passed to
            % fzero
            roots(end+1) = fzero(fun, [startingPoints(i) startingPoints(i+1)]);
            
            % NOTE: fzero will return an error if the function values at
            % the endpoints of the interval have the same sign. (And this
            % error will be skipped due to the 'try' here.) Hence it is
            % possible, for example, for the function fun to dip below zero
            % for a very short amount, as long as it goes down and comes
            % back up within the intervals specified. If that happens, then
            % we will accidently miss that mass which should have been
            % used to compute the integral.
        end
    end
    
    % Get rid of 0 and 1 as solutions
    roots = roots(0 < roots);
    roots = roots(roots < 1);
     
    if isempty(roots) == 1
        
        % Have to determine if the function is always negative or always
        % positive, to know what the value should be when there are no
        % roots
        
        % The domain of fun is (0,1)
        NtestVals = 100;
        testVals = rand(NtestVals,1);
        
        funTestVals = nan(length(testVals),1);
        
        for i=1:length(testVals)
            funTestVals(i) = fun(testVals(i));
        end
        
        funTestValsPos = (funTestVals > 0);

        if mean(funTestValsPos) < 0.5
            value = 1;
        elseif mean(funTestValsPos) > 0.5
            value = 0;
        else
            error('UBintegralPiece not working properly [no root case].');
            % If we get here then something strange is going on. The issue
            % discused below means that really mean(funTestValsPos) should
            % be either
            %
            % (1) equal to, or very close to 1, or
            % (2) equal to, or very close to 0.
            %
            % It should not be close to 0.5, or equal to it.
        end
        
        % An older version of the code simply tested the function at a
        % single point, and used that to determine whether the function was
        % completely above or below zero. The problem is that, as discussed
        % above, if the grid of intervals is not fine enough, the function
        % could dip down and come back up (or vice versa), before the
        % interval is over. 
        %
        % If this happens exactly at the single point of evaluation
        % then we would completely assign the wrong sign to
        % the integral. For this reason, the code here tests many
        % different values, and then we assign based on whether most of
        % these values are positive or negative.
        %
        % In principle, this approach could still fail, for example, if the
        % function dips below zero within all of the intervals of the
        % TestVals.
    
    else 
        %% Compute value of integral when there are multiple roots
        % Here we compute the value of the integral, based on partitioning the
        % function's domain into the parts where it is negative and the parts
        % where it is positive. For example, suppose the function starts out
        % negative, and crosses into the positive region at the first root.
        % Then we compute
        %
        % [root(1)-0] + [root(3)-root(2)] + [root(5)-root(4)] + ....
        %
        % Conversely, if the function started out positive, we compute
        %
        % [root(2) - root(1)] + [root(4) - root(3)] + ...
        %

        oddRoots = roots(1:2:end); % odd indexed roots
        evenRoots = roots(2:2:end); % even indexed roots
        
        tolerance = 1e-4;
        toleranceIncrement = 1e-4;
        valueNotAssigned = 1;
        
        UBintMaxIter = 100;
        
        UBintIter = 0;
        while valueNotAssigned == 1
            UBintIter = UBintIter + 1;
            
            if UBintIter >= UBintMaxIter
                
                warning('Switching from root finding to numerical integration in UBintegralPiece.m');
                
                forceNumericalInt = 1;
                
                value = UBintegralPiece(params,gridParams,z,c,...
                                    Data,h,[],[],forceNumericalInt);
                                
                valueNotAssigned = 0;
                
                continue;
            end
        
            if fun(roots(1)-tolerance) < 0 % Function starts out negative

                if mod(length(roots),2) == 0 % And has an even number of roots
                    value = sum(oddRoots) + 1 - sum(evenRoots);
                else
                    value = sum(oddRoots) - sum(evenRoots);
                end
                
                valueNotAssigned = 0;

            elseif fun(roots(1)-tolerance) > 0 % Function starts out positive

                if mod(length(roots),2) == 0
                    value = sum(evenRoots) - sum(oddRoots);
                else
                    value = sum(evenRoots) + 1 - sum(oddRoots);
                end
                
                valueNotAssigned = 0;
                
            elseif fun(roots(1)-tolerance) == 0 % Tolerance is too small to determine sign direction
                
                % Increase tolerance a little bit and return to the
                % beginning
                tolerance = tolerance + toleranceIncrement;
            end
            
        end
        UBintIter = 0; % This might be unnecessary, but just in case.
    
    end
    
    end % End of the numerical integration or not switch
    
end