function CI = computeBootstrapCI(params,cgrid,plowerbarGrid,vecToPass,...
                                        validCenteredVec,Nbootstrap,alpha)
%% Compute equal width uniform CIs
% 
% There are two approaches:
% (1) Build CIs based on the BC, and then truncate the CIs, or
% (2) Build CIs based on the BF from the get-go
%
% The variable useBC is an indicator for which approach to use. Our
% baseline approach is (1), and it is described as follows.
%
% The distribution of the elements of BCvecBootCentered approximates the
% distribution of
%
% sqrt{n} [ BChat(c) - BC(c) ]
%
% Hence in this and the following sections we just obtain the 1-alpha 
% quantile of this distribution and use it to construct confidence bands.

% Note: I use CI.uniformCrit to define a starting point for the unequal
% width bands

CI.supVec = nan(Nbootstrap,length(plowerbarGrid));
CI.uniformCrit = nan(length(plowerbarGrid),1);
CI.BFvec_CIuniform_onesided_lb = nan(length(cgrid),length(plowerbarGrid));

for j=1:length(plowerbarGrid)
    
    for b=1:Nbootstrap

        CI.supVec(b,j) = max(validCenteredVec(:,j,b));
        
    end

    CI.uniformCrit(j) = quantile(CI.supVec(:,j), 1-alpha);

    CI.BFvec_CIuniform_onesided_lb(:,j) = vecToPass(:,j) ...
        - CI.uniformCrit(j)/sqrt(params.sampleSize);
    
    % Next we truncate the band. This happens for two possible reasons:
    %
    % (1) Because we based things off the BC, which itself wasn't
    % truncated, or 
    %
    % (2) Because we are shifting the estimated curve down. This issue
    % arises even if useBC==0.
    %
    for k=1:length(cgrid)
        if CI.BFvec_CIuniform_onesided_lb(k,j) < 0
            CI.BFvec_CIuniform_onesided_lb(k,j) = 0;
        elseif CI.BFvec_CIuniform_onesided_lb(k,j) > 1
            CI.BFvec_CIuniform_onesided_lb(k,j) = 1;
        else
            % Don't change it's value
        end
    end
    
end

%% 2. Compute unequal width uniform CIs

% WARNING: Here I impose the fact that the grid points are equally spaced.
% This holds in the simulations and our empirical application,
% but in other uses of the code this assumption may not hold anymore.

[CI.kopt,CI.flag] = ...
    computeUnequalCritVals(vecToPass,validCenteredVec,CI.uniformCrit,cgrid,...
        params,plowerbarGrid,alpha);

%% Define the new confidence intervals

CI.BFvec_CIuniformUnequal_onesided_lb = nan(length(cgrid),length(plowerbarGrid));

for j=1:length(plowerbarGrid)
    
    % If the BF is degenerate then the band is just zero
    if max(vecToPass(:,j)) <= 0
        CI.BFvec_CIuniformUnequal_onesided_lb(:,j) = zeros(length(cgrid),1);
        continue;
    end

    CI.BFvec_CIuniformUnequal_onesided_lb(:,j) = vecToPass(:,j) ...
        - CI.kopt(:,j)/sqrt(params.sampleSize);
    
    for k=1:length(cgrid)
        if CI.BFvec_CIuniformUnequal_onesided_lb(k,j) < 0
            CI.BFvec_CIuniformUnequal_onesided_lb(k,j) = 0;
        elseif CI.BFvec_CIuniformUnequal_onesided_lb(k,j) > 1
            CI.BFvec_CIuniformUnequal_onesided_lb(k,j) = 1;
        else
            % Don't change its value
        end
    end
    
    % Monotonizing the CI
    for k=1:(length(cgrid)-1)
        if CI.BFvec_CIuniformUnequal_onesided_lb(k+1,j) > CI.BFvec_CIuniformUnequal_onesided_lb(k,j)
            CI.BFvec_CIuniformUnequal_onesided_lb(k+1,j) = CI.BFvec_CIuniformUnequal_onesided_lb(k,j);
        end
    end
        
end

CI.alpha = alpha;

end