% Really I don't need to pass plowerbarGrid here. I just need to know its
% length, which can be obtained from BFvecBootCentered
function [kopt,flag] = computeUnequalCritVals(BFvec,BFvecBootCentered,uniformCrit,cgrid,params,plowerbarGrid,alpha)

kopt = nan(length(cgrid),length(plowerbarGrid));
avgWidth = nan(length(plowerbarGrid),1);
flag = nan(length(plowerbarGrid),1);

for j=1:length(plowerbarGrid)
    
    % If BF is always zero or negative, then the estimated BF is
    % degenerate, and we are not going to compute confidence bands
    if max(BFvec(:,j)) <= 0
        kopt(:,j) = zeros(length(cgrid),1);
        flag(j) = 100;
        continue;
    end
    
    Dvec = squeeze(BFvecBootCentered(:,j,:));
    
    % Note: Sometimes we get degenerate distributions of Dvec, for fixed
    % c's. Think about the case where the BF is flat at the top for a long
    % time, so the truncation point binds. Then there is no variation at
    % all there. In this case the Hansen bandwidth outputs NaN. This
    % usually happens if the number of bootstrap draws is too small.
    % 
    % The next code addresses this issue: Check each row to see if there 
    % is any variation in Dvec. If not we use k(c) = 0
    
    DvecVariation = nan(length(cgrid),1);
    for i=1:length(cgrid)

        % Additional details: Why would we want to check max(Dev)>0? Dvec
        % is just the passed BFvecBootCentered, which is realizations of a
        % centered and scaled estimator, and hence should take BOTH
        % negative and positive values. But the issue is that if we pass
        % the BF, because of the truncation from above, we can get zeros in
        % BFvecBootCentered. But this doesn't happen when we pass
        % BCvecBootCentered instead.
        if params.useBC == 0
            if max(Dvec(i,:)) > 0
                DvecVariation(i) = 1;
            else
                DvecVariation(i) = 0;
            end
            
        elseif params.useBC == 1
            DvecVariation(i) = 1; % Otherwise it might remain NaN
        end
        
        % If BFvec is already zero, then even if there is variation in
        % Dvec, we will never use k(c) because the lower band will always
        % be below zero. So get rid of these components too. The <= here is
        % to accomodate BCvec and BFvec, both of which will yield the same
        % result here
        if BFvec(i,j) <= 0
            DvecVariation(i) = 0;
        end
        
    end
        
    kopt(DvecVariation == 0,j) = 0;

    Npositivekopt = sum(DvecVariation == 1);
    
    if Npositivekopt == 0
        warning('There is no variation in the bootstrap BF estimates.');
    end
    
    hvec = nan(Npositivekopt,1);
    hvecFull = nan(length(cgrid),1);
    
    % Compute Hansen bandwidths for each marginal distribution
    for i=1:length(cgrid)
        if DvecVariation(i) == 1
            hvecFull(i) = computeHansenBandwidth(Dvec(i,:)',4);
        end
    end
    clear i;
    
    hvec = hvecFull(DvecVariation == 1);
    
    for i=1:length(hvec)
        if isnan(hvec(i))
            hvec(i) = 0;
            warning('NaN in Hansen bandwidth; replacing with zero.');
        end
    end
    clear i;
    
    klb = zeros(Npositivekopt,1);
    kub = sqrt(params.sampleSize)*ones(Npositivekopt,1);
    % Any larger and we would subtract off more than 1, which is the 
    % logical range of the function.
    
    % Here I use the equal width band as the initial point
    kstart = repmat(uniformCrit(j),Npositivekopt,1);
    
    % It's possible that kstart is outside the logical bounds, so fix that
    % now:
    kstart = kub.*(kstart > kub) + kstart.*(kstart <= kub);
    % Note that since all entries are the same, either all violate the
    % bound or none do.
    
    fun = @(k) sum(k);
    
    nConst = ...
        @(k) bandConstraint(k,Dvec(DvecVariation==1,:),alpha,hvec);
    
    options = optimset('TolCon',1e-6,'MaxFunEvals',5e5,'MaxIter',5e5,'TolX',1e-6);
    
    % Note: We do not use the upper bound constraint here
    [koptshort, avgWidth(j),flag(j)] = fmincon(fun,kstart,[],[],[],[],klb,[],nConst,options);
    
    % If we fail to find a solution. In this case, do multi-start
    maxMultistarts = 10;
    mCounter = 0;
    
    while flag(j) <= 0
        display(mCounter);  
        
        if mCounter == maxMultistarts
            display('Multistart failed to find a solution.');
            break;
        end
        
        kNewStart = rand(length(kstart),1).*(kub-0);
            
        [koptshort, avgWidth(j),flag(j)] = ...
            fmincon(fun,kNewStart,[],[],[],[],klb,[],nConst,options);
        
        mCounter = mCounter + 1;
      
    end
        
    % Remember that this has to be divided by sqrt(sampleSize) before it is
    % subtracted from the estimated curve
    kopt(DvecVariation == 1,j) = koptshort;
    
end

end