% This code is adapted from BF_empirical_main_v2.m
function runEmpiricalMC_cluster(epsilon)
warning off optim:fminunc:SwitchingMethod;

useCluster = 1;

if useCluster == 0
    % For testing the code before submitting the entire job
    
    job_id_string = '285'; % '285' is any number between 1 and nSimulations
    job_id = str2double(job_id_string);

elseif useCluster == 1
    % For running jobs in parallel
    
    % Get the current job_id from the server.
    job_id_string = getenv('SLURM_ARRAY_TASK_ID');
    job_id = str2double(job_id_string);

end

% We will be looping over the different simulation datasets
simulationNumber = job_id;

diary(sprintf('../output/runEmpiricalMC_BF_eps%f_sim%d.txt',...
    epsilon,simulationNumber));

% Controls the bootstrap draws
rng(81349+job_id,'twister');

%% Load data

load(sprintf('../datasets/MCdatasetEmpirical_sim%d.mat',simulationNumber));

Data.WsingleSuppSize = length(unique(Data.Wsingle));

for i=1:Data.WsingleSuppSize
    WsuppPoint = i;

    params.pW(i) = mean( (Data.Wsingle == WsuppPoint) );

end

clear WsuppPoint i;

%% Set flags

Data.covariatesOn = 1;
params.covariatesOn = 1;
% There are two flags since not every function gets both params and Data
% passed to it all the time

if Data.covariatesOn ~= params.covariatesOn
    error('Covariate flags do not match.');
end

params.storeFbounds = 1; % Default is 1. This tells us to pre-compute
                         % Flower and Fupper on cgrid. Speeds up
                         % computations. But can't do this when computing
                         % ATE breakdown points, because that needs to work
                         % for arbitrary c values.

%% Estimate unconditional treatment probability

params.p1hat = mean(Data.X); % Probability of being treated
params.p0hat = 1 - params.p1hat;

params.sampleSize = length(Data.X);

%% Estimate conditional treatment probabilities, P(X=1 | W=w)
if params.covariatesOn == 1

    for i=1:Data.WsingleSuppSize
        WsuppPoint = i;

        params.p1hatCond(i) = mean(Data.X(Data.Wsingle == WsuppPoint));
        params.p0hatCond(i) = 1 - params.p1hatCond(i);

    end

    clear WsuppPoint i;

end

%% Compute bandwidths for unconditional (on W) cdfs

h1 = computeHansenBandwidth(Data.Y(Data.X == 1),4);
h0 = computeHansenBandwidth(Data.Y(Data.X == 0),4);

cdfBandwidth.x1 = h1/2;
% Here we manually undersmooth so as to better capture the local shape of
% the cdf, as can be seen visually

cdfBandwidth.x0 = h0;

clear h1 h0;

%% Compute bandwidths for conditional (on W) cdfs
if params.covariatesOn == 1
    
    for i=1:Data.WsingleSuppSize
        WsuppPoint = i;

        indices1 = ((Data.X == 1).*(Data.Wsingle == WsuppPoint) == 1);
        indices0 = ((Data.X == 0).*(Data.Wsingle == WsuppPoint) == 1);

        cdfBandwidth.x1cond(i) = ...
          computeHansenBandwidth(Data.Y(indices1),4);

        cdfBandwidth.x0cond(i) = ...
          computeHansenBandwidth(Data.Y(indices0),4);

    end

    % Again, we manually undersmooth to better capture the local shape,
    % as can be seen visually
    cdfBandwidth.x1cond = cdfBandwidth.x1cond/2;
    cdfBandwidth.x0cond = cdfBandwidth.x0cond/2;

    clear indices1 indices0 WsuppPoint i;

end

% Store the positive bandwidth, because in some places we set this to zero
cdfBandwidthPositive = cdfBandwidth;

%% Set tuning parameters

alphaGrid = [0.10 0.05];
alpha_ptIdentP1g0 = 0.05;
alpha_ATEbp = 0.05;

%% Set grid parameters

if params.covariatesOn == 1
    
    % Pointwise in w maximum
    cupperbar = max([params.p1hatCond(:)' params.p0hatCond(:)']);
    
    % Here we manually fix cupperbar. This speeds up the bootstrap code
    cupperbar = 0.2;
    
elseif params.covariatesOn == 0 
    
    cupperbar = max(params.p1hat,params.p0hat);
    
end

% Number of intervals used when finding roots in computing the integral for
% LBintegralPiece. This needs to be sufficiently large so that we pick up
% all true roots. Results can be very wrong if this is too small.
gridParams.Ngrid = 100;

% We are interested in P(Y1 - Y0 >= z) where z is:
z = 0;

% Number of grid points for computing the breakdown frontier
Ncgrid = 20;

cgrid = linspace(0,cupperbar,Ncgrid);

% These are used in UBintegralPiece.m for the bootstrap (used to do
% numerical integration)
gridParams.NgridIncrement = 40;
gridParams.NIntgrid = 500;

% Choose which claims about the parameter we are interested in: 
%
% P(Y1 >= Y0) >= plowerbar
%
plowerbarGrid = [0.1 0.25 0.5 0.75 0.9];

Nbootstrap = 1000; 
Nbootstrap_ptIdentP1g0 = 0;
Nbootstrap_ATEbp = 0;

fprintf(1,'Number of cgrid points = %d.\n',Ncgrid);
fprintf(1,'Number of BF bootstrap samples = %d.\n',Nbootstrap);

%% Estimate breakdown frontier

params.p1 = params.p1hat;
params.p0 = params.p0hat;

if params.covariatesOn == 1
    params.p1cond = params.p1hatCond;
    params.p0cond = params.p0hatCond;
end

[BFvec,intPieceVec,minPieceVec] = ...
    computeBFvec(cgrid,plowerbarGrid,params,gridParams,z,Data,cdfBandwidth);

clear params.p1 params.p0;

trunc.top = 0;
trunc.bottom = 0;
BCvec = computeBCvec(intPieceVec,minPieceVec,plowerbarGrid,trunc);

%% Bootstrap
rng(130913,'twister');

if Nbootstrap > 0

    % Use empirical cdf for bootstrap draws
    % These must be set to zero even if we have covariates in the analysis
    cdfBandwidth.x1 = 0;
    cdfBandwidth.x0 = 0;
    
    if params.covariatesOn == 1
        for i=1:Data.WsingleSuppSize
            cdfBandwidth.x1cond(i) = 0;
            cdfBandwidth.x0cond(i) = 0;
        end
    end

    [BFvecBootCentered, BFvecBoot, BCvecBoot, BCvecBootCentered, ...
      intPieceVecBoot, minPieceVecBoot, minPieceVecBootNotMono] ...
        = computeBootstrapVecs(params,gridParams,z,cdfBandwidth,cgrid,...
                  plowerbarGrid,Data,BFvec,BCvec,Nbootstrap,epsilon,trunc);
              
    params.useBC = 1; % Decide how to compute CIs: based on the BC or BF
    
    % The difference here is whether we use the BC or BF as our baseline
    if params.useBC == 1
        vecToPass = BCvec;
        validCenteredVec = BCvecBootCentered;
    elseif params.useBC == 0
        vecToPass = BFvec;
        validCenteredVec = BFvecBootCentered;
    end
    
    for i=1:length(alphaGrid)
        alpha = alphaGrid(i);

        CI(i) = computeBootstrapCI(params,cgrid,plowerbarGrid,...
                              vecToPass,validCenteredVec,Nbootstrap,alpha);

    end

elseif Nbootstrap == 0
    
    for i=1:length(alphaGrid)
        
        CI(i).BFvec_CIuniformUnequal_onesided_lb = ...
            zeros(length(cgrid),length(plowerbarGrid));
        
    end
    
end

%% Variables which can be reconstructed
% To keep file sizes small

% These just need B*vecBoot, B*vec, and epsilon to construct
clear BFvecBootCentered BCvecBootCentered;

% These just need intPieceVecBoot,minPieceVecBoot,plowerbar
clear BFvecBoot BCvecBoot;

% These four vectors are the worst culprits, since they are 
% (Ncgrid by NplowerbarGrid by Nbootstrap)

clear vecToPass validCenteredVec;

%% Save output
clear i ans job_id job_id_string;

save(sprintf('../MCestimates/BF_EmpiricalMCestimates_eps%2.4f_sim%d.mat',...
    epsilon,simulationNumber),'-v7.3');

diary('off');

end