function runMC_cluster(dgpChoice,sampleSize,epsilon)
warning off optim:fminunc:SwitchingMethod;

useCluster = 1;

if useCluster == 0
    % Use this for testing the code before submitting the entire job
    
    testParams.saveBeforeCI = 1;
    
    dgpChoice = 1;
    sampleSize = 500; 
    epsilon = 0.4472;
    
    job_id_string = '14'; % '14' is any number between 1 and nSimulations
    job_id = str2double(job_id_string);

elseif useCluster == 1
    % for running jobs in parallel
    
    testParams.saveBeforeCI = 0;
    
    % Get the current job_id from the server.
    job_id_string = getenv('SLURM_ARRAY_TASK_ID');
    job_id = str2double(job_id_string);

end

% We will be looping over the different simulation datasets
simulationNumber = job_id;

diary(sprintf('../output/runMC_BF_dgp%d_n%d_eps%f_sim%d.txt',...
    dgpChoice,sampleSize,epsilon,simulationNumber));

% Controls the bootstrap draws
rng(81349+job_id,'twister');

%% Load data

if useCluster == 0

    load(sprintf('../datasets/MCdataset_dgp%d_n%d_sim%d.mat',...
        dgpChoice,sampleSize,simulationNumber));
    
elseif useCluster == 1

    load(sprintf('../datasets/MCdataset_dgp%d_n%d_sim%d.mat',...
        dgpChoice,sampleSize,simulationNumber));
    
end

%% Set flags

Data.covariatesOn = 0;
params.covariatesOn = 0;
% There are two flags since not every function gets both params and Data
% passed to it all the time

if Data.covariatesOn ~= params.covariatesOn
    error('Covariate flags do not match.');
end

params.storeFbounds = 0; % Default is 1. This tells us to pre-compute
                         % Flower and Fupper on cgrid. Speeds up
                         % computations. But can't do this when computing
                         % ATE breakdown points, because that needs to work
                         % for arbitrary c values.

%% Set sample size parameter

params.sampleSize = sampleSize;

%% Set grid parameters

run setGridParams.m;

% Number of intervals used when finding roots in computing the integral for
% LBintegralPiece. This needs to be sufficiently large so that we pick up
% all true roots. Results can be very wrong if this is too small.
gridParams.Ngrid = 40; % Use 100 for empirical application


if gridParams.Ngrid < 2
    error('Ngrid should be at least 2.');
end 

gridParams.NgridIncrement = 40;
gridParams.NIntgrid = 500;

Nbootstrap = 1000;
% Set = 0 to turn off
% Also, if it's >0 but too small then the CI optimization may have problems

fprintf(1,'Number of bootstrap samples = %d.\n',Nbootstrap);

%% Set tuning parameters

alphaGrid = [0.05];

%% Estimate treatment probability

params.p1hat = mean(Data.X); % Probability of being treated
params.p0hat = 1 - params.p1hat;

%% Compute bandwidths

h1 = computeHansenBandwidth(Data.Y(Data.X == 1),4);
h0 = computeHansenBandwidth(Data.Y(Data.X == 0),4);

cdfBandwidth.x1 = h1;
cdfBandwidth.x0 = h0;

clear h1 h0;

%% Estimate BF

params.p1 = params.p1hat;
params.p0 = params.p0hat;

[BFvec,intPieceVec,minPieceVec] = ...
    computeBFvec(cgrid,plowerbarGrid,params,gridParams,z,Data,cdfBandwidth);

clear params.p1 params.p0;

trunc.top = 0;
trunc.bottom = 0;
BCvec = computeBCvec(intPieceVec,minPieceVec,plowerbarGrid,trunc);

%% Bootstrap

if Nbootstrap > 0
    
    % Use empirical cdf when computing the BFs in the bootstrap samples
    cdfBandwidth.x1 = 0;
    cdfBandwidth.x0 = 0;

    [BFvecBootCentered, BFvecBoot, BCvecBoot, BCvecBootCentered, ...
      intPieceVecBoot, minPieceVecBoot, minPieceVecBootNotMono] ...
        = computeBootstrapVecs(params,gridParams,z,cdfBandwidth,cgrid,...
                  plowerbarGrid,Data,BFvec,BCvec,Nbootstrap,epsilon,trunc);

    % Sometimes the CI computations mess up, so here I just save the
    % bootstrap data before that step.
    if testParams.saveBeforeCI == 1
        save(sprintf('../MCestimates/BF_MCestimates_dgp%d_n%d_eps%2.4f_sim%d.mat',...
            dgpChoice,sampleSize,epsilon,simulationNumber),'-v7.3');
    end
    
    params.useBC = 1; % Decide how to compute CIs: based on the BC or BF

    % The difference here is whether we use the BC or BF as our baseline
    if params.useBC == 1
        vecToPass = BCvec;
        validCenteredVec = BCvecBootCentered;
    elseif params.useBC == 0
        vecToPass = BFvec;
        validCenteredVec = BFvecBootCentered;
    end

    for i=1:length(alphaGrid)
        alpha = alphaGrid(i);

        CI(i) = computeBootstrapCI(params,cgrid,plowerbarGrid,...
                              vecToPass,validCenteredVec,Nbootstrap,alpha);

    end

end

%% Remove variables which can be reconstructed
% To keep file sizes small

% These just need B*vecBoot, B*vec, and epsilon to construct
clear BFvecBootCentered BCvecBootCentered;

% These just need intPieceVecBoot,minPieceVecBoot,plowerbar
clear BFvecBoot BCvecBoot;

% These four vectors are the worst culprits, since they are 
% (Ncgrid by NplowerbarGrid by Nbootstrap)

clear vecToPass validCenteredVec;

%% Save results

save(sprintf('../MCestimates/BF_MCestimates_dgp%d_n%d_eps%2.4f_sim%d.mat',...
    dgpChoice,sampleSize,epsilon,simulationNumber),'-v7.3');

diary('off');

end