% This function inverts the smoothed cdf estimator. It does this as
% follows:
% 
% 1. Compute the smoothed cdf at a grid of y points.
%
% 2. These give us a grid tau points
%
% 3. For the specific tau at which we want Q_Y(tau), use linear
% interpolation on the grid of points from part 2.
%
% 4. If we are unable to interpolate, this is because we're close to the
% boundary. So fix that ex post.
%
function val = smoothedQuantile(tau,x,Data,h,WsuppPoint,DataBootstrap,epsilon)

    % Extend definition of the function to entire support in a logical way
    if tau < 0
        tau = 0;
    elseif tau > 1
        tau = 1;
    end

    % NOTE: Even if we want to use covariates, we must set the
    % unconditional bandwidths to zero if we want to use empirical
    % quantiles here.
    
    % Use empirical conditional quantile
    if (h.x0 == 0) || (h.x1 == 0)
        if nargin == 4
            val = quantile(Data.Y(Data.X==x),tau);
            
        elseif nargin == 5
            
            indicator = ((Data.X == x).*(Data.Wsingle == WsuppPoint) == 1); 
            val = quantile(Data.Y(indicator),tau);
            
        elseif nargin == 7
            
            val = computePerturbedQuantile(tau,x,WsuppPoint,DataBootstrap,Data.covariatesOn);
            
        end
        
    % Use smoothed quantile
    else

        Nygrid = 100;
        ygrid = linspace(min(Data.Y),max(Data.Y),Nygrid);

        tauvals = nan(1,length(ygrid));

        for i=1:length(ygrid)
            y = ygrid(i);

            if nargin == 4
                tauvals(i) = smoothedConditionalCDF(y,x,Data,h);
                
            elseif nargin == 5
                tauvals(i) = smoothedConditionalCDF(y,x,Data,h,WsuppPoint);

            elseif nargin == 7
                tauvals(i) = ...
                  smoothedConditionalCDF(y,x,Data,h,WsuppPoint,DataBootstrap,epsilon);                    

            end

        end

        % This can be NaN if tau is outside of tauvals. However, in that
        % case the next few lines reset val appropriately.        
        val = interp1(tauvals, ygrid, tau);
        
        %% When the desired quantile doesn't appear in the range of the cdf
        if tau < min(tauvals)
            val = min(Data.Y);

        elseif tau > max(tauvals)
            val = max(Data.Y);
        end

    end
    
    if isnan(val)
        error('Quantile inversion not working correctly.');
    end

end